# -*-python-*-

__package__    = "s60-fladdressbook"
__version__    = "0.11"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/fladdressbook/"
__cvsversion__ = "$Revision: 1.7 $"
__date__       = "$Date: 2006/08/22 18:22:33 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

import e32
import appuifw
from elementtree.ElementTree import XML
from pdis.xpath import compile

import httplib

if not e32.in_emulator() :
    import sysinfo
    import contacts

#
#
#
    
class FladdressBook:
    
    def __init__(self):
        self.lock = e32.Ao_lock()

        self.old_title = appuifw.app.title
        appuifw.app.title = u"FladdressBook"

        self.exit_flag = False
        appuifw.app.exit_key_handler = self.abort

        self.__msglog = u""
        self.__flickr = u"www.flickr.com"
        self.__apikey = u"787acbb6cee3864e303499ea7a36f5d0"

        self.__options = [(u"Merge a contact", self.merge_contact)]
        
    #
    #
    #
    
    def loop(self):

        if self.setup() :
            self.run();
            
        self.lock.wait()

    #
    #
    #

    def setup (self) :

        if not self.load_contact_list() :
            return False

        if not self.test_connection() :
            return False

        if not self.test_api_key() :
            return False
        
        appuifw.app.menu = self.__options
        return True

    #
    #
    #

    def test_connection (self) :

        self.log("Testing connection");
        
        if not e32.in_emulator() :
            if sysinfo.signal() == 0 :
                self.error("No carrier signal")
                return False

        head = self.execute_http_request(self.__flickr, '/', 'HEAD')

        if head == False :
            self.error("Unable to connect to flickr.com")
            return False
        
        return True

    #
    #
    #

    def test_api_key (self) :

        self.log("Testing API key");
        
        xml = self.api_call('flickr.test.echo')

        if xml == None :
            return False
        
        xpath = compile("/rsp")
        meth  = xpath.evaluate(xml)

        return True
    
    #
    #
    #

    def run (self) :
        self.clear_log()
        
        self.log("Your address book has been loaded.")
        self.log("To merge a contact's Flickr account select 'Merge a contact' from the Options menu.")
        self.log("--------------------------------")
        
    #
    #
    #
    
    def abort (self) :
        self.kill = True
        self.old_title = appuifw.app.title

        self.lock.signal()

    #
    #
    #

    def load_contact_list (self) :

        if e32.in_emulator() :
            return True
        
        self.log(u"Loading contacts")

        db = contacts.open()
        self.db = db
        
        self.contacts = []
        self.contacts_key = {}
        counter = 0

        menu_key  = {}
            
        for id in db :
            person = db[id]
            first  = person.find('first_name')
            last   = person.find('last_name')        
            
            if not len(first) and not len(last) :
                continue
            
            if len(first) :
                first = first[0].value
            else :
                first = None
                    
            if len(last) :
                last = last[0].value
            else :
                last = None

            if (last) :
                str_name = last
                if (first) :
                    str_name += ", "

            if (first) :
                str_name += first
            
            self.contacts.append(str_name)
            menu_key[str_name] = id

        self.contacts.sort()
        
        for str_name in self.contacts :
            self.contacts_key[counter] = menu_key[str_name]
            counter += 1

        return True
        
    #
    #
    #

    def choose_contact (self) :

        if e32.in_emulator() :
            return None
        
        idx = appuifw.popup_menu(self.contacts)

        if idx == None :
            return None
        
        return self.db[self.contacts_key[idx]]

    #
    #
    #
    
    def choose_contact_email (self, contact) :

        if e32.in_emulator() :
            return appuifw.query(u"Please enter an email address", "text")
            
        if contact == None :
            return None
    
        addrs_list = contact.find('email_address')
        merge_addr = ""
    
        if len(addrs_list) > 1 :

            choices = []

            for addr in addrs_list :
                choices.append(addr.value)

            idx = appuifw.popup_menu(choices)

            if idx == None :
                return idx
            
            merge_addr = choices[idx]
        
        else :
            merge_addr = addrs_list[0].value

        return merge_addr

    #
    #
    #
    
    def merge_contact(self) :

        contact = self.choose_contact()
        email   = self.choose_contact_email(contact)
        
        if email == None :
            self.log("No email address provider, oh well...")
            return

        nsid = self.get_nsid(email)

        if nsid == None :
            self.error("No NSID for that email was found!")
            return False
        
        info = self.get_user_info(nsid)

        if info == None :
            self.error("Foiled again!")            
            return false
        
        if e32.in_emulator():
            self.dump_info(info)
            return True
        
        try :
            self.merge_data(contact, info)
        except Exception, e :
            self.error("Hrm, failed to merge Flickr data : %s" % e)
            return False

        self.log("Flickr data successfully merged!")
        return True

    #
    #
    #

    def dump_info(self, info) :

        for label, value in info.items() :
            self.log("[%s] %s" % (label, value))

        return True
    
    #
    #
    #

    def merge_data(self, contact, info) :
        
        contact.begin()

        self.set_field(contact, 'url', 'Flickr Photos', info['photosurl'])
        self.set_field(contact, 'url', 'Flickr Photos (Mobile)', info['mobileurl'])        
        self.set_field(contact, 'url', 'Flickr RSS Feed', info['rssurl'])
        self.set_field(contact, 'note', 'Flickr NSID', info['nsid'])        
        self.set_field(contact, 'note', 'Flickr Username', info['username'])
        self.set_field(contact, 'note', 'Flickr FOAF mbox_sha1sum', info['mbox_sha1sum'])        
        
        contact.commit()
        return True
    
    #
    #
    #
    
    def set_field (self, contact, stype, slabel, svalue) :
        
        for field in contact.find(type=stype) :

            if field.label == slabel :
                field.value = svalue
                return

        contact.add_field(stype, value=svalue, label=slabel)
        
    #
    #
    #

    def get_nsid(self, email) :
        self.log("Fetching Flickr NSID for '%s'" % email)
        
        xml = self.api_call('flickr.people.findByEmail', find_email=email)

        xpath = compile("/rsp/user")
        user  = xpath.evaluate(xml)[0]

        return user.attrib['nsid']

    #
    #
    #

    def get_user_info(self, nsid) :
        self.log("Fetching user info from Flickr")
        
        xml = self.api_call('flickr.people.getInfo', user_id=nsid)

        xpath  = compile("/rsp/person")
        person = xpath.evaluate(xml)[0]

        # http://static.flickr.com/{icon-server}/buddyicons/{nsid}.jpg

        rss_url = 'http://www.flickr.com/services/feeds/photos_public.gne?id=' + nsid + '&format=rss_200'
        
        return {
            'nsid'         : nsid,
            'buddyicon'    : '',
            'username'     : person.find("username").text,
            'photosurl'    : person.find("photosurl").text,
            'rssurl'       : rss_url,
            'mobileurl'    : person.find("mobileurl").text,
            'mbox_sha1sum' : person.find("mbox_sha1sum").text
            }

    #
    #
    #

    def api_call(self, method, **kwargs) :

        params = ['method=' + method,
                  'api_key=' + self.__apikey]
    
        for key in kwargs :
            params.append('='.join([key, kwargs[key]]))
        
        url = '/services/rest/?' + '&'.join(params)

        res = self.execute_http_request(self.__flickr, url)

        if res == None :
            return None
        
        raw = res.read()

        try :
            xml = XML(raw)
        except Exception, e :
            self.log("XML parse error : %s" % e)
            return None

        if self.is_api_error(xml) :
            return None

        return xml

    #
    #
    #
    
    def is_api_error (self, xml) :

        xp  = compile("/rsp")
        rsp = xp.evaluate(xml)[0]

        if rsp.attrib['stat'] == 'fail' :
            err = rsp.find('err')
            self.log("API error : %s (%s)" % (err.attrib['code'], err.attrib['msg']))
            return True

        return False
    
    #
    #
    #
    
    def execute_http_request (self, endpoint, url, method='GET') :

        try :
            conn = httplib.HTTPConnection(endpoint)
            conn.request(method, url)
        except Exception, e :
            self.log("HTTP request failed : %s" % e)
            return None
            
        return conn.getresponse()
        
    #
    #
    #

    def error (self, msg) :
        self.log("[error] %s" % msg)

    #
    #
    #
    
    def log (self, msg) :

        self.__msglog += "%s\n" % msg
        
        t = appuifw.Text()
        t.write(unicode(self.__msglog))

        appuifw.app.body = t

    #
    #
    #
    
    def clear_log (self) :
        self.__msglog = u""
        
#
#
#
    
if __name__ == "__main__" :
    app = FladdressBook()
    app.loop()
